Package: miaDash
Version: 1.2.0
Authors@R: 
    c(person(given = "Giulio", family = "Benedetti", role = c("aut", "cre"),
           email = "giulio.benedetti@utu.fi",
           comment = c(ORCID = "0000-0002-8732-7692")),
    person(given = "Akewak", family = "Jeba", role = c("ctb"),
             email = "akewak.k.jeba@utu.fi",
             comment = c(ORCID = "0009-0007-1347-7552")),
    person(given = "Leo", family = "Lahti", role = c("aut"),
             email = "leo.lahti@iki.fi",
             comment = c(ORCID = "0000-0001-5537-637X")))
Title: Dashboard for the interactive analysis and exploration of
        microbiome data
Description: 
    miaDash provides a Graphical User Interface for the exploration of
    microbiome data. This way, no knowledge of programming is required to
    perform analyses. Datasets can be imported, manipulated, analysed and
    visualised with a user-friendly interface.
biocViews: Microbiome, Software, Visualization, GUI, ShinyApps,
        DataImport
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.4.0), iSEE (>= 2.19.4), shiny
Imports: ape, bluster, htmltools, iSEEtree (>= 1.1.4), mia, rintrojs,
        scater, scuttle, shinydashboard, shinyjs, shinyWidgets,
        S4Vectors, SingleCellExperiment, SummarizedExperiment,
        TreeSummarizedExperiment, utils, vegan
Suggests: BiocStyle, knitr, philr, remotes, rmarkdown, testthat (>=
        3.0.0)
URL: https://github.com/microbiome/miaDash
BugReports: https://github.com/microbiome/miaDash/issues
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/miaDash
git_branch: RELEASE_3_22
git_last_commit: b230dcf
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:04:09 UTC; biocbuild
Author: Giulio Benedetti [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8732-7692>),
  Akewak Jeba [ctb] (ORCID: <https://orcid.org/0009-0007-1347-7552>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>)
Maintainer: Giulio Benedetti <giulio.benedetti@utu.fi>

# Generated by roxygen2: do not edit by hand

export(miaDash)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(TreeSummarizedExperiment,TreeSummarizedExperiment)
importFrom(TreeSummarizedExperiment,rowTree)
importFrom(ape,read.tree)
importFrom(bluster,DmmParam)
importFrom(bluster,HclustParam)
importFrom(bluster,KmeansParam)
importFrom(bluster,NNGraphParam)
importFrom(htmltools,HTML)
importFrom(htmltools,br)
importFrom(htmltools,div)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(iSEE,iSEE)
importFrom(iSEEtree,.check_all_panels)
importFrom(iSEEtree,AbundanceDensityPlot)
importFrom(iSEEtree,AbundancePlot)
importFrom(iSEEtree,ColumnGraphPlot)
importFrom(iSEEtree,ColumnTreePlot)
importFrom(iSEEtree,LoadingPlot)
importFrom(iSEEtree,PrevalencePlot)
importFrom(iSEEtree,RDAPlot)
importFrom(iSEEtree,RowGraphPlot)
importFrom(iSEEtree,RowTreePlot)
importFrom(iSEEtree,ScreePlot)
importFrom(mia,addAlpha)
importFrom(mia,addCluster)
importFrom(mia,addHierarchyTree)
importFrom(mia,addPrevalence)
importFrom(mia,addPrevalentAbundance)
importFrom(mia,agglomerateByRank)
importFrom(mia,getDissimilarity)
importFrom(mia,getTaxonomyLabels)
importFrom(mia,importBIOM)
importFrom(mia,importHUMAnN)
importFrom(mia,importMetaPhlAn)
importFrom(mia,importMothur)
importFrom(mia,importQIIME2)
importFrom(mia,runNMDS)
importFrom(mia,runRDA)
importFrom(mia,subsetByPrevalent)
importFrom(mia,subsetByRare)
importFrom(mia,taxonomyRanks)
importFrom(mia,transformAssay)
importFrom(rintrojs,introjs)
importFrom(scater,runMDS)
importFrom(scater,runPCA)
importFrom(scuttle,addPerCellQC)
importFrom(shinyWidgets,addSpinner)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardBody)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(stats,as.formula)
importFrom(utils,data)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(vegan,vegdist)

# miaDash <img src="inst/assets/mia_logo.png" align="right" width="120" />

[![issues](https://img.shields.io/github/issues/microbiome/miaDash)](https://github.com/microbiome/miaDash/issues)
[![pulls](https://img.shields.io/github/issues-pr/microbiome/miaDash)](https://github.com/microbiome/miaDash/pulls)
[![R-CMD-check](https://github.com/microbiome/miaDash/workflows/build/badge.svg)](https://github.com/microbiome/miaDash/actions)
[![codecov](https://codecov.io/gh/microbiome/miaDash/branch/devel/graph/badge.svg)](https://app.codecov.io/gh/microbiome/miaDash?branch=devel)
[![codefactor](https://www.codefactor.io/repository/github/microbiome/miadash/badge)](https://www.codefactor.io/repository/github/microbiome/miadash)

The goal of miaDash is to provide a user-friendly interface to import,
manipulate, analyse and visualise TreeSummarizedExperiment objects.

## Usage

miaDash is available online at [this address](https://miadash-microbiome.2.rahtiapp.fi/).
While suitable for small and medium datasets, the online version may slow down
when larger datasets are analysed (> 500 samples). In this case, the app can
be installed and run locally. Either way, functionality to subset and
agglomerate the data is also provided in the app.

## Installation instructions

The release version can be installed from Bioconductor as follows:

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("miaDash")
```

Contributors or users interested in the latest functionality can install the
devel version as follows:

```
remotes::install_github("microbiome/miaDash")
```

## Example

The basic functionality of miaDash can be explored as follows:

```
# Import miaDash
library(miaDash)

# Launch miaDash
if (interactive()) {
  miaDash()
}
```

## Code of Conduct

Please note that the miaDash project is released with a
[Contributor Code of Conduct](https://bioconductor.org/about/code-of-conduct/).
By contributing to this project, you agree to abide by its terms. Contributions
are welcome in the form of feedback, issues and pull requests. You can find the
contributor guidelines of the miaverse
[here](https://github.com/microbiome/mia/blob/devel/CONTRIBUTING.md).

## Acknowledgements

miaDash results from the joint effort of the larger R/Bioconductor community. In
particular, this software mainly depends on the following packages:

- [_mia_](https://bioconductor.org/packages/release/bioc/html/mia.html)
- [_iSEEtree_](https://bioconductor.org/packages/devel/bioc/html/iSEEtree.html)
- [_iSEE_](https://www.bioconductor.org/packages/release/bioc/html/iSEE.html)
- [TreeSummarizedExperiment](https://www.bioconductor.org/packages/release/bioc/html/TreeSummarizedExperiment.html)
- [_shiny_](https://cran.r-project.org/web/packages/shiny/)

#' Landing page
#' 
#' \code{.landing_page} creates the landing page of miaDash, where TreeSE objects
#' can be built and iSEE can be launched.
#'
#' @return The UI is defined by the function. A \code{NULL} value is invisibly
#'   returned.
#'
#' @name landing_page
#' @keywords internal

#' @rdname landing_page
#' @importFrom shinydashboard dashboardPage dashboardHeader dashboardSidebar
#'   dashboardBody box
#' @importFrom htmltools HTML br tags div tagList
#' @importFrom shinyjs disable
#' @importFrom shinyWidgets addSpinner
#' @importFrom utils data
.landing_page <- function(FUN, input, output, session) {
  
    # nocov start
    mia_datasets <- .import_datasets(-c(2, 5))
  
    output$allPanels <- renderUI({

        dashboardPage(
          
            dashboardHeader(disable = TRUE),
            dashboardSidebar(disable = TRUE),
            dashboardBody(
                
                tags$head(tags$style(HTML(".btn-primary {color: white}"))),
              
                fluidRow(box(id = "import.panel", title = "Import", width = 4,
                    status = "primary", solidHeader = TRUE, collapsible = TRUE,

                    tabsetPanel(id = "format",
                          
                        tabPanel(title = "Dataset", value = "dataset", br(),
                        
                            selectInput(inputId = "data", label = "Dataset:",
                                choices = mia_datasets,
                                selected = mia_datasets[1])),
                          
                        tabPanel(title = "R Object", value = "rds", br(),
                                   
                            fileInput(inputId = "file", label = "RDS:",
                                accept = ".rds", placeholder = "object.rds")),
                          
                        tabPanel(title = "Raw Data", value = "raw", br(),
                                   
                            fileInput(inputId = "assay", label = "Assays:",
                                accept = ".tsv", multiple = TRUE,
                                placeholder = "assay.tsv"),
                            div(style = "margin-top: -20px"),
                                   
                            fileInput(inputId = "coldata", label = "colData:",
                                accept = ".tsv", placeholder = "coldata.tsv"),
                            div(style = "margin-top: -20px"),
                        
                            fileInput(inputId = "rowdata", label = "rowData:",
                                accept = ".tsv", placeholder = "rowdata.tsv"),
                            div(style = "margin-top: -20px"),
                            
                            fileInput(inputId = "row.tree",
                                  label = "rowTree:", placeholder = "row.tree",
                                  accept = c(".tree", ".tre")),
                            div(style = "margin-top: -20px"),
                
                            fileInput(inputId = "col.tree",
                                  label = "colTree:", placeholder = "col.tree",
                                  accept = c(".tree", ".tre")),
                            div(style = "margin-top: -20px"),
                            
                            checkboxInput(inputId = "taxa.from.rownames",
                                label = "Derive rowData from assay rownames")),
                          
                        tabPanel(title = "Foreign", value = "foreign", br(),
                                  
                            radioButtons(inputId = "ftype", label = "Type:",
                                choices = list("biom", "HUMAnN", "MetaPhlAn",
                                "Mothur", "QIIME2"), inline = TRUE),
                             
                            fileInput(inputId = "main.file",
                                label = "Main file:", accept = c(".biom",
                                ".tsv", ".shared", ".QZA", ".txt"),
                                placeholder = "biom, tsv, shared, QZA or txt"),
                            div(style = "margin-top: -20px"),
                            
                            fileInput(inputId = "col.data", label = "colData:",
                                accept = c(".tsv", ".design"),
                                placeholder = "tsv or design"),
                            div(style = "margin-top: -20px"),
                            
                            conditionalPanel(
                                condition = paste0("input.ftype == 'Mothur' | ",
                                    "input.ftype == 'QIIME2'"),
                               
                                fileInput(inputId = "f.rowdata",
                                    label = "rowData:", accept = c(".taxonomy",
                                    ".qza"), placeholder = "taxonomy or qza"),
                                div(style = "margin-top: -20px")),
                                
                            conditionalPanel(
                                condition = paste0("input.ftype == 'biom' | ",
                                    "input.ftype == 'MetaPhlAn'"),
                                
                                fileInput(inputId = "tree.file",
                                    label = "rowTree:", placeholder = "tree.tree",
                                    accept = c(".tree", ".tre", ".qza")),
                                div(style = "margin-top: -20px")),
                            
                            conditionalPanel(
                                condition = paste0("input.ftype == 'biom' | ",
                                    "input.ftype == 'HUMAnN'"),
                            
                                checkboxInput(inputId = "rm.tax.pref",
                                    label = "Remove taxa prefixes")),
                                
                            conditionalPanel(
                                condition = "input.ftype == 'biom'",
                            
                                checkboxInput(inputId = "rank.from.pref",
                                    label = "Derive taxa from prefixes")),
                            
                            conditionalPanel(
                                condition = "input.ftype == 'HUMAnN'",
                            
                                checkboxInput(inputId = "rm.hum.suf",
                                    label = "Remove sample suffix")))),
              
                    actionButton("import", "Upload", class = "btn-primary")),
            
                box(id = "manipulate.panel", title = "Manipulate", width = 4,
                    status = "primary", solidHeader = TRUE, collapsible = TRUE,

                    tabsetPanel(id = "manipulate",
                          
                        tabPanel(title = "Subset", value = "subset", br(),
                      
                            radioButtons(inputId = "subkeep", label = "Keep:",
                                choices = c("prevalent", "rare"),
                                inline = TRUE),
                      
                            selectInput(inputId = "subassay", label = "Assay:",
                                choices = NULL),
                      
                            sliderInput(inputId = "prevalence", value = 0,
                                label = "Prevalence threshold:", step = 0.01,
                                min = 0, max = 1),
                      
                            numericInput(inputId = "detection", value = 0,
                                label = "Detection threshold:", min = 0,
                                step = 1)),
                  
                        tabPanel(title = "Agglomerate", value = "agglomerate",

                            br(),
                  
                            selectInput(inputId = "taxrank",
                                label = "Taxonomic rank:", choices = NULL)),
                  
                        tabPanel(title = "Transform", value = "transform", br(),
                  
                            selectInput(inputId = "assay.type",
                                label = "Assay:", choices = NULL),
              
                            selectInput(inputId = "trans.method",
                                label = "Method:", choices = .transformMethods),
              
                            checkboxInput(inputId = "pseudocount",
                                label = "Pseudocount"),
              
                            textInput(inputId = "assay.name", label = "Name:"),
                      
                            radioButtons(inputId = "margin", label = "Margin:",
                                choices = c("samples", "features"),
                                inline = TRUE))),
              
                    actionButton("apply", "Apply", class = "btn-primary")),
            
                box(id = "estimate.panel", title = "Estimate", width = 4,
                    status = "primary", solidHeader = TRUE, collapsible = TRUE,

                    tabsetPanel(id = "estimate",
                                
                        header = tagList(
                            br(), selectInput(inputId = "estimate.assay",
                                label = "Assay:", choices = NULL)),
                        
                        tabPanel(title = "Quality", value = "quality",
                            
                            checkboxGroupInput(inputId = "quality.metrics",
                                label = "Metrics:",
                                choices = .qualityMetrics)),
                          
                        tabPanel(title = "Alpha", value = "alpha",
                      
                            selectInput(inputId = "alpha.index",
                                label = "Metrics:", multiple = TRUE,
                                choices = .alphaMetrics)),

                        tabPanel(title = "Beta", value = "beta",
                           
                            radioButtons(inputId = "bmethod",
                                label = "Method:", choices = .betaMethods,
                                inline = TRUE),
                      
                            conditionalPanel(
                                condition = paste0("input.bmethod == 'MDS' || ",
                                    "input.bmethod == 'NMDS' || ",
                                    "input.bmethod == 'RDA'"),
                            
                                selectInput(inputId = "beta.index",
                                    label = "Metric:", choices = .betaMetrics)),
                      
                            conditionalPanel(
                                condition = "input.bmethod == 'RDA'",
                            
                                textInput(inputId = "rda.formula",
                                    label = "Formula:",
                                    placeholder = "data ~ var1 + var2 * var3")),
                      
                            numericInput(inputId = "ncomponents", value = 5,
                                label = "Number of components:", min = 1,
                                step = 1)),
                        
                        tabPanel(title = "Cluster", value = "cluster",
                                 
                            radioButtons(inputId = "cmethod",
                                label = "Method:", choices = .clustMethods,
                                inline = TRUE),
                            
                            conditionalPanel(
                                condition = paste0("input.cmethod == 'Dmm' | ",
                                    "input.cmethod == 'Kmeans'"),
                                
                                numericInput(inputId = "kclusters", value = 3,
                                    label = "Number of clusters:", min = 1,
                                    step = 1)),
                            
                            conditionalPanel(
                                condition = "input.cmethod == 'Dmm'",
                              
                                selectInput(inputId = "dmm.type",
                                    label = "Criterion:",
                                    choices = .DmmCriteria,
                                    selected = .DmmCriteria[1]),
                                
                                numericInput(inputId = "dmm.seed", value = 1L,
                                    label = "Seed:", min = 0, step = 1)),
                            
                            conditionalPanel(
                                condition = "input.cmethod == 'NNGraph'",
                              
                                numericInput(inputId = "kneighbours",
                                    value = 10, label = "Number of neighbours:",
                                    min = 1, step = 1),
                                
                                checkboxInput(inputId = "nn.shared",
                                    label = "Construct shared graph")),
                            
                            checkboxInput(inputId = "clust.full",
                                label = "Add to metadata"),
                            
                            radioButtons(inputId = "clust.margin",
                                label = "Margin:", inline = TRUE,
                                choices = c("samples", "features"),
                                selected = "features")),
                        
                        footer = textInput(inputId = "estimate.name",
                            label = "Name:")),
              
                    actionButton("compute", "Compute", class = "btn-primary"))),
                
                fluidRow(box(id = "visualise.panel", title = "Visualise",
                    width = 4, status = "primary", solidHeader = TRUE,
                    collapsible = TRUE,

                    selectInput(inputId = "panels", label = "Panels:",
                        choices = c(.miaDashDefaultPanels, .miaDashOtherPanels),
                        multiple = TRUE, selected = c(.miaDashDefaultPanels)),
                
                    actionButton("launch", "Launch iSEE",
                        class = "btn-primary")),
              
                box(id = "output.panel", title = "Output", width = 8,
                    status = "primary", solidHeader = TRUE, collapsible = TRUE,

                    addSpinner(verbatimTextOutput(outputId = "object"),
                        color = "#007bff"),
              
                    downloadButton(outputId = "download", label = "Download",
                        class = "btn-primary")))))})
    
    ## Disable navbar buttons that are not linked to any observer yet
    disable("iSEE_INTERNAL_organize_panels")  # organize panels
    disable("iSEE_INTERNAL_link_graph")       # link graph
    disable("iSEE_INTERNAL_export_content")   # export content
    disable("iSEE_INTERNAL_tracked_code")     # tracked code
    disable("iSEE_INTERNAL_panel_settings")   # panel settings
    disable("iSEE_INTERNAL_open_vignette")    # open vignette
    disable("iSEE_INTERNAL_session_info")     # session info
    disable("iSEE_INTERNAL_citation_info")    # citation info
    
    rObjects <- reactiveValues(tse = NULL)
    
    observe({
        .print_message(
            title = "Welcome to the Microbiome Analysis Dashboard! \U0001f9a0",
            "miaDash is actively maintained by the",
            tags$a(href = "https://datascience.utu.fi/",
            "Turku Data Science Group", target = "_blank", .noWS = "after"),
            ", so we are happy to receive feedback from you. Feature requests,",
            "bug reports and other comments can be submitted",
            tags$a(href = "https://github.com/microbiome/miaDash/issues",
            "here", target = "_blank", .noWS = "after"), HTML(".<br/><br/>"),
            "If you are new to this app, you can learn how to use it with",
            tags$a(href = "https://microbiome.github.io/miaDash/articles/miaDash.html",
            "this short tutorial", target = "_blank", .noWS = "after"),
            ". Technical support can be obtained on",
            tags$a(href = "https://app.gitter.im/#/room/#microbiome_miaverse:gitter.im",
            "our Gitter channel", target = "_blank", .noWS = "after"), "." 
        )
    })
    
    .create_import_observers(input, rObjects)
    .create_manipulate_observers(input, rObjects)
    .create_estimate_observers(input, rObjects)
    .update_observers(input, session, rObjects)

    .create_launch_observers(FUN, input, session, rObjects)
    
    .render_overview(output, rObjects)
    .render_download(output, rObjects)

    invisible(NULL)
    # nocov end
}

#' Outputs
#'
#' \code{.render_overview} and \code{.render_download} create the output of the
#'   UI, which consists of the overview of the TreeSE object and the download
#'   object, respectively.
#'
#' @param output The Shiny output object from the server function.
#'
#' @return Adds a rendered item to \code{output}.
#'   A \code{NULL} value is invisibly returned.
#'
#' @name render_output
#' @keywords internal

#' @rdname render_output
.render_overview <- function(output, rObjects) {
  
    # nocov start
    output$object <- renderPrint({
        rObjects$tse
    })
    # nocov end

    invisible(NULL)
}

#' @rdname render_output
.render_download <- function(output, rObjects) {
  
    # nocov start
    output$download <- downloadHandler(
        filename = function() paste0("se-", Sys.Date(), ".rds"),
        content = function(file) saveRDS(rObjects$tse, file)
    )
    # nocov end
  
    invisible(NULL)
}

test_that("outputs", {

    output <- new.env()
    rObjects <- new.env()
  
    overview_out <- .render_overview(output, rObjects)
    download_out <- .render_download(output, rObjects)
  
    expect_null(overview_out)
    expect_null(download_out)
    expect_named(output, c("object", "download"))
    
})

.landing_page_tour <- rbind(
    data.frame(
        element="#import\\.panel",
        intro=paste(
          "First, you need to import a dataset from files. Several formats are",
          "supported. Alternatively, you can experiment on miaDash with one of",
          "the ready-made <a href='https://microbiome.github.io/mia/reference/mia-datasets.html'>mia datasets</a>.",
          "<br/><br/>When ready, click on the Upload button to import the",
          "dataset as a TreeSummarizedExperiment object."
        )
    ),
    data.frame(
        element="#manipulate\\.panel",
        intro=paste(
            "You may need to wrangle the data a little bit to get the best",
            "visualisations out of it. Here, you can apply operations such as",
            "subsetting, agglomeration and transformation to the dataset.",
            "<br/><br/>After setting the proper parameters, click on the Apply",
            "button to perform the operation on the dataset."
        )
    ),
    data.frame(
        element="#estimate\\.panel",
        intro=paste(
            "Here, you can perform standard microbiome analysis to estimate",
            "alpha (within-sample) and beta (between-sample) diversity with",
            "several methods.<br/><br/>Click on the Compute button to apply",
            "selected method to the dataset."
        )
    ),
    data.frame(
        element="#visualise\\.panel",
        intro=paste(
            "Here you can select which panels to use to visualise the dataset.",
            "Check the <a href='https://bioconductor.org/packages/devel/bioc/vignettes/iSEEtree/inst/doc/iSEEtree.html#12_Panels'>iSEEtree catalogue</a>",
            "to find the right panel.<br/><br/>When ready, click the Launch",
            "button to explore the dataset."
        )
    ),
    data.frame(
        element="#output\\.panel",
        intro=paste(
            "Here you see the TreeSummarizedExperiment object you created.",
            "Every time you press the Upload, Apply or Compute buttons, new",
            "elements are added to the object.<br/><br/>You can download the",
            "object as an RDS file by clicking the Download button."
        )
    )
)

#' miaDash
#'
#' miaDash is a web app that provides an interface to build and explore
#' \code{\link[TreeSummarizedExperiment:TreeSummarizedExperiment-constructor]{TreeSummarizedExperiment}}
#' (TreeSE) objects by means of \link[iSEE:iSEE]{iSEE}.
#'
#' @return An \code{\link[iSEE:iSEE]{iSEE}} app with a custom landing page to
#'   build TreeSE objects and explore \link[mia:mia-datasets]{mia datasets}.
#'
#' @examples
#' app <- miaDash()
#'
#' if (interactive()) {
#'   shiny::runApp(app)
#' }
#' 
#' @seealso \link[iSEE:iSEE]{iSEE} \link[mia:mia]{mia}
#'   \link[miaViz:miaViz]{miaViz}
#'
#' @name miaDash

#' @export
#' @rdname miaDash
#' @importFrom iSEE iSEE
#' @importFrom utils packageVersion
#' @importFrom htmltools tags
miaDash <- function() {
    
    addResourcePath("assets", system.file("assets", package = "miaDash"))
  
    iSEE(landingPage = .landing_page, tabTitle = "miaDash", appTitle = tags$div(
        paste0("Microbiome Analysis Dashboard - v", packageVersion("miaDash")),
        tags$img(src = "assets/mia_logo.png", height = "40px", style = "margin-left: 10px"),
        style = "cursor: pointer; font-weight: 500",
        onclick = "window.location='https://miadash-microbiome.2.rahtiapp.fi/'"))
}

#' @importFrom shinyjs enable
#' @importFrom iSEEtree .check_all_panels RowTreePlot RowTreePlot AbundancePlot
#'   AbundanceDensityPlot RDAPlot ScreePlot LoadingPlot ColumnTreePlot
#'   RowGraphPlot ColumnGraphPlot PrevalencePlot
.launch_isee <- function(FUN, initial, session, rObjects) {

    # nocov start
    tse <- rObjects$tse
  
    initial <- lapply(initial, function(x) eval(parse(text = paste0(x, "()"))))
    initial <- .check_all_panels(tse, initial)
  
    FUN(SE = tse, INIT = initial)
  
    enable("iSEE_INTERNAL_organize_panels")  # organize panels
    enable("iSEE_INTERNAL_link_graph")       # link graph
    enable("iSEE_INTERNAL_export_content")   # export content
    enable("iSEE_INTERNAL_tracked_code")     # tracked code
    enable("iSEE_INTERNAL_panel_settings")   # panel settings
    enable("iSEE_INTERNAL_open_vignette")    # open vignette
    enable("iSEE_INTERNAL_session_info")     # session info
    enable("iSEE_INTERNAL_citation_info")    # citation info
  
    invisible(NULL)
    # nocov end
}

#' miaDash utilities
#' 
#' Helper functions to support the app functionality.
#' 
#' @param selection \code{Numeric vector}. A list of indices for the mia
#'   datasets to return.
#' 
#' @param tse a
#' \code{\link[TreeSummarizedExperiment:TreeSummarizedExperiment-constructor]{TreeSummarizedExperiment}}
#' object.
#' 
#' @param fun \code{Function scalar}. Function to apply to \code{tse}.
#' 
#' @param fun.args \code{Named list}. A list of arguments to pass to \code{fun}.
#' 
#' @param title \code{Character scalar}. The title of the error message to
#'   print. (Default: \code{"Invalid input:"})
#' 
#' @param item \code{Character scalar}. The file path from which the object
#'   should be loaded.
#' 
#' @param loader \code{Function scalar}. The function to load \code{item}.
#'   (Default: \code{NULL})
#' 
#' @param alternative an alternative output to return when at least one
#'   \code{item} and \code{loader} are not defined. (Default: \code{NULL})
#' 
#' @param form \code{Character scalar} The formula to be checked.
#' 
#' @param ... Either a series of strings to form the message returned by
#'   \code{.print_message} or named arguments for \code{loader}.
#'
#' @return
#' \itemize{
#' \item \code{.import_datasets}: returns the list of available mia datasets.
#' \item \code{.update_tse}: returns a TreeSE object after applying \code{fun}
#'   with arguments \code{fun.args}. Eventual messages and errors are also
#'   printed.
#' \item \code{.print_message}: returns a modalDialog with the error message
#'   specified with \code{...} and titled \code{title}.
#' \item \code{.set_optarg}: returns an optional element for a TreeSE
#'   constructor. The output is either an object located at the file path
#'   \code{item} and loaded with \code{loader} or \code{alternative}.
#' \item \code{.check_formula}: returns \code{TRUE} or \code{FALSE} depending
#'   whether or not all variables included in \code{form} are present in
#'   \code{se} colData.
#' }
#'
#' @keywords internal
#' @name utils
NULL

#' @rdname utils
.import_datasets <- function(selection) {
  
    mia_datasets <- data(package = "mia")
    mia_datasets <- mia_datasets$results[selection, "Item"]
    data(list = mia_datasets, package = "mia")
    
    return(mia_datasets)
}

#' @rdname utils
.update_tse <- function(tse, fun, fun.args = list()) {

    tse <- tryCatch({withCallingHandlers({
        
        do.call(fun, fun.args)
      
        # nocov start
        }, message = function(m) {
        
            showNotification(conditionMessage(m))
            invokeRestart("muffleMessage")
        
        })}, error = function(e) {
      
            .print_message(e, title = "Unexpected error:")
            return(tse)
          
        })
        # nocov end
  
    return(tse)
}

#' @rdname utils
.print_message <- function(..., title = "Invalid input:") {

    # nocov start
    showModal(modalDialog(
        title = title, ...,
        easyClose = TRUE, footer = NULL
    ))
    # nocov end
}

#' @rdname utils
.set_optarg <- function(item, loader = NULL, alternative = NULL, ...){
  
    if( !(is.null(item) || is.null(loader)) ){
        out <- loader(item, ...)
    } else {
        out <- alternative
    }
  
    return(out)
}

#' @rdname utils
#' @importFrom SummarizedExperiment colData
.check_formula <- function(form, tse){
  
    form <- gsub("data ~\\s*", "", form)
    vars <- unlist(strsplit(form, "\\s*[\\+|\\*]\\s*"))
  
    cond <- all(vars %in% names(colData(tse)))
    return(cond)
}

#' @rdname utils
#' @importFrom mia importBIOM
#' @importFrom SummarizedExperiment colData
#' @importFrom TreeSummarizedExperiment rowTree
#' @importFrom S4Vectors DataFrame
#' @importFrom ape read.tree
#' @importFrom utils read.table
.importBIOM <- function(file, col.data = NULL, tree.file = NULL, ...){
  
    tse <- importBIOM(file, ...)
  
    if( !is.null(col.data) ){
        coldata <- read.table(file = col.data, header = TRUE, sep = "\t")
        rownames(coldata) <- colnames(tse)
        colData(tse) <- DataFrame(coldata)
    }

    if( !is.null(tree.file) ){
        rowTree(tse) <- read.tree(tree.file)
    }
  
    return(tse)
}

#' @rdname utils
#' @importFrom SummarizedExperiment rowData
#' @importFrom S4Vectors DataFrame
#' @importFrom mia getTaxonomyLabels
.rownames2taxa <- function(x){

    tax_df <- data.frame(Taxonomy = rownames(x))
      
    rowdata <- mia:::.parse_taxonomy(
        tax_df,
        col.name = "Taxonomy",
        removeTaxaPrefixes = TRUE
    )

    rowData(x) <- DataFrame(rowdata)
    rownames(x) <- getTaxonomyLabels(x, make.unique = TRUE)
    
    return(x)
}

test_that("utils", {
  
    data("Tengeler2020", package = "mia")
    tse <- Tengeler2020

    idx <- c(1, 3)
    expect_equal(.import_datasets(idx),
        data(package = "mia")$results[idx, "Item"])
    
    expect_no_error(
        tse <- .update_tse(tse, transformAssay,
            list(x = tse, assay.type = "counts", method = "relabundance"))
    )
    
    item <- NULL
    expect_identical(.set_optarg(item, alternative = "alternative"),
       "alternative")
    
    item <- "100"
    expect_identical(.set_optarg(item, loader = as.numeric), 100)
  
    expect_true(.check_formula("data ~ patient_status + cohort", tse))
    expect_false(.check_formula("data ~ wrong_var + sample_name", tse))
})

#' Constants
#'
#' Constant values used throughout the miaDash app.
#' 
#' @section Panel layout:
#' \describe{
#' \item{\code{.miaDashDefaultPanels}}{List of panel names in the default layout of miaDash.}
#' \item{\code{.miaDashOtherPanels}}{List of panel names not in the default layout of miaDash.}
#' \item{\code{.transformMethods}}{List of transformations applicable to assays.}
#' \item{\code{.alphaMetrics}}{List of alpha diversity metrics.}
#' \item{\code{.betaMetrics}}{List of beta diversity metrics.}
#' \item{\code{.betaMethods}}{List of beta diversity methods.}
#' \item{\code{.qualityMetrics}}{List of metrics for quality control.}
#' }
#'
#' @author Giulio Benedetti
#' 
#' @keywords internal
#' @name constants
#' @aliases .miaDashDefaultPanels .miaDashOtherPanels .transformMethods
#' .qualityMetrics .alphaMetrics .betaMetrics .betaMethods .clustMethods
#' .DmmCriteria
NULL

#' @rdname constants
.miaDashDefaultPanels <- c("RowDataTable", "ColumnDataTable", "RowTreePlot",
    "AbundancePlot", "AbundanceDensityPlot", "ReducedDimensionPlot",
    "ComplexHeatmapPlot")

#' @rdname constants
.miaDashOtherPanels <- c("PrevalencePlot", "RDAPlot", "ScreePlot",
    "LoadingPlot", "ColumnTreePlot", "RowGraphPlot", "ColumnGraphPlot",
    "RowDataPlot", "ColumnDataPlot")

#' @rdname constants
.transformMethods <- c("alr", "chi.square", "clr", "css", "frequency",
    "hellinger", "log", "log10", "log2", "max", "normalize", "pa",# "philr",
    "range", "rank", "rclr", "relabundance", "rrank", "standardize", "total")

#' @rdname constants
.qualityMetrics <- list("Library size" = "PerCellQC",
    "Prevalence" = "Prevalence", "Prevalent abundance" = "PrevalentAbundance",
    "Hierarchy tree" = "HierarchyTree")

#' @rdname constants
.alphaMetrics <- c("coverage_diversity", "fisher_diversity", "faith_diversity",
    "gini_simpson_diversity", "inverse_simpson_diversity",
    "log_modulo_skewness_diversity", "shannon_diversity", "absolute_dominance",
    "dbp_dominance", "core_abundance_dominance", "gini_dominance",
    "dmn_dominance", "relative_dominance", "simpson_lambda_dominance",
    "camargo_evenness", "pielou_evenness", "simpson_evenness", "evar_evenness",
    "bulla_evenness", "ace_richness", "chao1_richness", "hill_richness",
    "observed_richness")

#' @rdname constants
.betaMetrics <- c("euclidean", "bray", "jaccard", "unifrac")

#' @rdname constants
.betaMethods <- c("MDS", "NMDS", "PCA", "RDA")
#"TSNE", "UMAP")

#' @rdname constants
.clustMethods <- c("Dmm", "Hclust", "Kmeans", "NNGraph")

#' @rdname constants
.DmmCriteria <- c("laplace", "AIC", "BIC")

#' Observers
#' 
#' \code{.create_observers} and \code{.create_launch_observers} define the
#' server to import and build TreeSE objects and track the state of the Build
#' and Launch buttons.
#'
#' @param input The Shiny input object from the server function.
#' @param rObjects A reactive list of values generated in the landing page.
#'
#' @return Observers are created in the server function in which this is called.
#' A \code{NULL} value is invisibly returned.
#'
#' @name create_observers
#' @keywords internal

#' @rdname create_observers
#' @importFrom utils read.table
#' @importFrom ape read.tree
#' @importFrom S4Vectors DataFrame
#' @importFrom mia importHUMAnN importMetaPhlAn importQIIME2 importMothur
#'   addAlpha
#' @importFrom TreeSummarizedExperiment TreeSummarizedExperiment
.create_import_observers <- function(input, rObjects) {

    # nocov start
    observeEvent(input$import, {
      
        if( input$format == "dataset" ){
      
            rObjects$tse <- isolate(get(input$data))
      
        }else if( input$format == "rds" ){
      
            isolate({
                req(input$file)
                rObjects$tse <- readRDS(input$file$datapath)
            })
      
        }else if( input$format == "raw" ){
      
            isolate({
                req(input$assay)
                
                assay_list <- lapply(input$assay$datapath,
                    function(x) as.matrix(read.table(x, row.names = 1,
                        header = TRUE, sep = "\t")))
                
                names(assay_list) <- gsub(".tsv", "", input$assay$name)
                
                coldata <- .set_optarg(input$coldata$datapath,
                    alternative = DataFrame(row.names = colnames(assay_list[[1]])),
                    loader = read.table, row.names = 1, header = TRUE, sep = "\t")

                rowdata <- .set_optarg(input$rowdata$datapath,
                    loader = read.table, row.names = 1,
                    header = TRUE, sep = "\t")
               
                row.tree <- .set_optarg(input$row.tree$datapath,
                    loader = read.tree)
                
                col.tree <- .set_optarg(input$col.tree$datapath,
                    loader = read.tree)
                
                fun_args <- list(assays = assay_list, colData = coldata,
                    rowData = rowdata, rowTree = row.tree, colTree = col.tree)
        
                rObjects$tse <- .update_tse(
                     rObjects$tse, TreeSummarizedExperiment, fun_args
                )
                
                if( input$taxa.from.rownames ){
                  
                    rObjects$tse <- .update_tse(
                        rObjects$tse, .rownames2taxa, list(x = rObjects$tse)
                    )
                    
                }
            
            })
      
        }else if( input$format == "foreign" ){
          
            isolate({
                req(input$main.file)
              
                coldata <- .set_optarg(input$col.data$datapath,
                    alternative = input$col.data$datapath)
                
                treefile <- .set_optarg(input$tree.file$datapath,
                    alternative = input$tree.file$datapath)
                
                if( input$ftype == "biom" ){

                    fun_args <- list(file = input$main.file$datapath,
                        col.data = coldata, tree.file = treefile,
                        removeTaxaPrefixes = input$rm.tax.pref,
                        rankFromPrefix = input$rank.from.pref)
                    
                    rObjects$tse <- .update_tse(
                        rObjects$tse, .importBIOM, fun_args
                    )
              
                }else if( input$ftype == "HUMAnN" ){
                
                    fun_args <- list(file = input$main.file$datapath,
                        col.data = coldata,
                        prefix.rm = input$rm.tax.pref,
                        remove.suffix = input$rm.hum.suf)
                    
                    rObjects$tse <- .update_tse(
                        rObjects$tse, importHUMAnN, fun_args
                    )

                }else if( input$ftype == "MetaPhlAn" ){

                    fun_args <- list(file = input$main.file$datapath,
                        col.data = coldata, tree.file = treefile)
              
                    rObjects$tse <- .update_tse(
                        rObjects$tse, importMetaPhlAn, fun_args
                    )
                
                }else if( input$ftype %in% c("Mothur", "QIIME2") ){
                  
                    imp_fun <- eval(parse(text = paste0("import", input$ftype)))
                
                    rowdata <- .set_optarg(input$f.rowdata$datapath,
                        alternative = input$f.rowdata$datapath)
                    
                    fun_args <- list(assay.file = input$main.file$datapath,
                        row.file = input, col.file = rowdata)
                    
                    rObjects$tse <- .update_tse(
                        rObjects$tse, imp_fun, fun_args
                    )
                
                }
        
            })
            
        }
      
    }, ignoreInit = TRUE, ignoreNULL = FALSE)
    # nocov end
  
    invisible(NULL)
}

#' @rdname create_observers
#' @importFrom SummarizedExperiment assay
#' @importFrom mia subsetByPrevalent subsetByRare agglomerateByRank
#'   transformAssay
.create_manipulate_observers <- function(input, rObjects) {
  
    # nocov start
    observeEvent(input$apply, {
      
        if( input$manipulate == "subset" ){
          
            isolate({
                req(input$subassay)
              
                if( input$subkeep == "prevalent" ){
                    subset_fun <- subsetByPrevalent
                } else if( input$subkeep == "rare" ){
                    subset_fun <- subsetByRare
                }
            
                fun_args <- list(x = rObjects$tse, assay.type = input$subassay,
                    prevalence = input$prevalence, detection = input$detection)
                
                rObjects$tse <- .update_tse(rObjects$tse, subset_fun, fun_args)
              
            })
          
        }
      
        else if( input$manipulate == "agglomerate" ){
          
            isolate({
                
                fun_args <- list(x = rObjects$tse, rank = input$taxrank)
                rObjects$tse <- .update_tse(
                     rObjects$tse, agglomerateByRank, fun_args
                )
              
            })
          
        } else if( input$manipulate == "transform" ){
          
            isolate({
                req(input$assay.type)
              
                if( input$assay.name != "" ){
                    name <- input$assay.name
                } else {
                    name <- input$trans.method
                }
              
                fun_args <- list(x = rObjects$tse, name = name,
                    method = input$trans.method, assay.type = input$assay.type,
                    MARGIN = input$margin, pseudocount = input$pseudocount)
                
                #if( input$trans.method == "philr" ){
                #    fun_args <- c(fun_args, tree = list(rowTree(rObjects$tse)),
                #        node.labels = rowTree(tse)$node.label)
                #}
                
                rObjects$tse <- .update_tse(
                     rObjects$tse, transformAssay, fun_args
                )
                
            })
          
        }
      
    }, ignoreInit = TRUE, ignoreNULL = TRUE)
    # nocov end
  
    invisible(NULL)
}

#' @rdname create_observers
#' @importFrom stats as.formula
#' @importFrom mia addAlpha runNMDS runRDA getDissimilarity addHierarchyTree
#'   addPrevalence addPrevalentAbundance addCluster
#' @importFrom TreeSummarizedExperiment rowTree
#' @importFrom scater runMDS runPCA
#' @importFrom scuttle addPerCellQC
#' @importFrom vegan vegdist
#' @importFrom bluster KmeansParam DmmParam HclustParam NNGraphParam
.create_estimate_observers <- function(input, rObjects) {
  
    # nocov start
    observeEvent(input$compute, {
        
        if( input$estimate == "quality" ){
        
            isolate({
                req(input$estimate.assay)
        
                for( qmetric in input$quality.metrics ){
                
                    qfun <- eval(parse(text = paste0("add", qmetric)))
                    
                    qfun_args <- list(x = rObjects$tse,
                        assay.type = input$estimate.assay)
                
                    rObjects$tse <- .update_tse(rObjects$tse, qfun, qfun_args)
                
                }
        
            })
        
        }else if( input$estimate == "alpha" ){
        
            if( is.null(input$alpha.index) ){
                .print_message("Please select one or more metrics.")
                return()
            }
        
            isolate({
                req(input$estimate.assay)
            
                if( input$estimate.name != "" ){
                    name <- input$estimate.name
                } else {
                    name <- input$alpha.index
                }
            
                fun_args <- list(x = rObjects$tse, name = name,
                    assay.type = input$estimate.assay, index = input$alpha.index)
                
                rObjects$tse <- .update_tse(rObjects$tse, addAlpha, fun_args)
          
            })
        
        }else if( input$estimate == "beta" ){
          
            if( input$ncomponents > nrow(rObjects$tse) - 1 ){
              
                .print_message(
                    "Please use a number of components smaller than the number",
                    "of features in the assay."
                )
              
                return()
            }
          
            isolate({
                req(input$estimate.assay)
              
                if( input$estimate.name != "" ){
                    name <- input$estimate.name
                }else{
                    name <- input$bmethod
                }
              
                beta_args <- list(x = rObjects$tse,
                    assay.type = input$estimate.assay,
                    ncomponents = input$ncomponents, name = name)
              
                if( input$beta.index == "unifrac" ){
                  
                    if( is.null(rowTree(rObjects$tse)) ){
                        .print_message("Unifrac cannot be computed without a rowTree.")
                        return()
                    }
                  
                    beta_args <- c(beta_args, FUN = getDissimilarity,
                        tree = list(rowTree(rObjects$tse)),
                        ntop = nrow(rObjects$tse), method = input$beta.index)
                    
                }else if( input$bmethod %in% c("MDS", "NMDS") ){
                  
                    beta_args <- c(beta_args, FUN = vegdist,
                        method = input$beta.index)
                    
                }else if( input$bmethod == "RDA" ){
                  
                    if( input$rda.formula == "" ){
                        .print_message("Please enter a formula.")
                        return()
                    }
                  
                    if( !.check_formula(input$rda.formula, rObjects$tse) ){
                        .print_message("Please make sure all elements in the",
                           "formula match variables of the column data.")
                        return()
                    }
                  
                    beta_args <- c(beta_args,
                        formula = as.formula(input$rda.formula))
                }
                
                beta_fun <- eval(parse(text = paste0("run", input$bmethod)))
                
                rObjects$tse <- .update_tse(rObjects$tse, beta_fun, beta_args)
            })
        
        }else if( input$estimate == "cluster" ){
            
            isolate({
                req(input$estimate.assay)
                
                if( input$estimate.name != "" ){
                    name <- input$estimate.name
                }else{
                    name <- "clusters"
                }
                
                if( input$cmethod == "Dmm" ){
                
                    blus_params <- list(k = input$kclusters,
                        type = deparse(input$dmm.type))#, seed = input$dmm.seed)
                
                }else if( input$cmethod == "Hclust" ){
                
                    blus_params <- list()
                
                }else if( input$cmethod == "Kmeans" ){
                
                    blus_params <- list(centers = input$kclusters)
                
                }else if( input$cmethod == "NNGraph" ){
                
                    blus_params <- list(shared = input$nn.shared,
                        k = input$kneighbours)
                
                }
                
                blus_params <- sprintf("%s=%s", names(blus_params), blus_params)
                blus_params <- paste(blus_params, collapse = ", ")
                blus_fun <- sprintf("%sParam(%s)", input$cmethod, blus_params)
                
                clust_args <- list(x = rObjects$tse,
                    assay.type = input$estimate.assay,
                    by = input$clust.margin, full = input$clust.full, 
                    BLUSPARAM = eval(parse(text = blus_fun)),
                    name = name, clust.col = name)
                
                rObjects$tse <- .update_tse(
                    rObjects$tse, addCluster, clust_args)
            })
            
        }
        
    }, ignoreInit = TRUE, ignoreNULL = TRUE)
    # nocov end
  
    invisible(NULL)
}

#' @rdname create_observers
#' @importFrom SummarizedExperiment assayNames
#' @importFrom mia taxonomyRanks
#' @importFrom rintrojs introjs
.update_observers <- function(input, session, rObjects){
  
    # nocov start
    observe({
      
      if( isS4(rObjects$tse) ){
        
          updateSelectInput(session, inputId = "subassay",
              choices = assayNames(rObjects$tse))
        
          updateSelectInput(session, inputId = "taxrank",
              choices = taxonomyRanks(rObjects$tse))
          
          updateSelectInput(session, inputId = "assay.type",
              choices = assayNames(rObjects$tse))
          
          updateSelectInput(session, inputId = "estimate.assay",
              choices = assayNames(rObjects$tse))
          
          updateNumericInput(session, inputId = "ncomponents",
              max = nrow(rObjects$tse) - 1)
        
      }
    
    })
    
    observeEvent(input$iSEE_INTERNAL_tour_steps, {
      
        introjs(session, options = list(steps = .landing_page_tour))
      
    }, ignoreInit = TRUE)
    # nocov end
    
    invisible(NULL)
}

#' @rdname create_observers
.create_launch_observers <- function(FUN, input, session, rObjects) {
  
    # nocov start
    observeEvent(input$launch, {
    
        .launch_isee(FUN, input$panels, session, rObjects)

    }, ignoreInit = TRUE, ignoreNULL = TRUE)
    # nocov end
  
    invisible(NULL)
}

test_that("observers", {
  
  input <- new.env()
  rObjects <- new.env()
  FUN <- function(SE, INITIAL) invisible(NULL)
  
  import_out <- .create_import_observers(input, rObjects)
  expect_null(import_out)
  
  manipulate_out <- .create_manipulate_observers(input, rObjects)
  expect_null(manipulate_out)
  
  estimate_out <- .create_estimate_observers(input, rObjects)
  expect_null(estimate_out)
  
  update_out <- .update_observers(input, session = NULL, rObjects)
  expect_null(update_out)
  
  launch_out <- .create_launch_observers(FUN, input, session = NULL, rObjects)
  expect_null(launch_out)
  
})

---
title: "miaDash"
author: 
  - name: Giulio Benedetti
    affiliation: University of Turku
    email: giulio.benedetti@utu.fi
output:
    BiocStyle::html_document:
        fig_height: 7
        fig_width: 10
        toc: true
        toc_float: true
        toc_depth: 2
        number_sections: true
        self_contained: true
        code_folding: show
bibliography: ../inst/REFERENCES.bib
date: "`r doc_date()`"
package: "`r pkg_ver('miaDash')`"
vignette: >
  %\VignetteIndexEntry{miaDash}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

This notebook provides a practical introduction to the Microbiome Analysis
Dashboard (miaDash), an interactive app to analyse and explore microbiome data.
Feel free to try it online at
[this address](miadash-microbiome.2.rahtiapp.fi/) with your data or one of
the ready-to-use example datasets. Here, its usage and functionality are
described in more detail.

## Motivation

Most of the tools available for microbiome data analysis require some knowledge
of programming. This represents a burden for practitioners more interested
in getting results than learning how to program. To this end, miaDash aims to
make microbiome analysis accessible to anyone who needs it, with or without any
computational skills.

As a word of caution, while the app removes the burden of programming, it is
still critical to understand the nature of microbiome data and how it can be
analysed. Such knowledge can be acquired from the online book [Orchestrating
Microbiome Analysis](https://microbiome.github.io/OMA/) (OMA) and several other
independent resources. The following section presents what is currently possible
in the app.

## Interface

The interface is divided into five tabs that reflect the steps of a typical
microbiome analysis pipeline. First, the dataset of interest can be uploaded
through the Import tab, where several data types and file formats are supported.
Alternatively, one of the available example datasets can be used for practice.
Second, a set of operations can be applied to the dataset through the
Manipulate tab, which include methods for subsetting features by prevalence,
agglomerating by taxonomic level and transforming assays. Third, the dataset
can be analysed through the Estimate tab, which provide common techniques to
quantify alpha and beta diversity. Finally, results can be explored through the
Visualise tab, where an interactive explorer can be launched with a customisable
set of panels that illustrate different aspects of the data. The full catalogue
of available panels can be found
[here](https://microbiome.github.io/iSEEtree/articles/panels.html).

# Tutorial

The app can be used online or locally, depending on resource availability and
size of the dataset to analyse. In general, the online version is freely
available, so that data of any type can be tested there. However, running the
app locally might be a better option for larger datasets (> 1000 features). In
this case, you may also consider subsetting and/or agglomerating the data.

## Installation

If you decided to run the app locally, make sure to have R installed in your
machine and execute the following command:

```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("miaDash")
```

Once the package is successfully installed, you should have access to the
development version of miaDash.

## Example

This section shows how to get started with miaDash. If you are using it locally,
run the next code chunk to launch the app. Otherwise, you can skip it.

```{r start, message=FALSE, warning=FALSE}
# Import miaDash
library(miaDash)

# Launch miaDash
if (interactive()) {
  miaDash()
}
```

```{r screenfun, eval=!exists("SCREENSHOT"), include=FALSE}
SCREENSHOT <- function(x, ...) knitr::include_graphics(x)
```

As described in Section [1.2](#interface), The dashboard consists of five
different windows with tools to import, manipulate, analyse and visualise
the dataset of choice. After launching the app, it appears as follows:

```{r screenplot1, echo=FALSE, out.width="100%"}
SCREENSHOT("screenshots/dashboard.png", delay = 20)
```

At first, the variety of options might feel intimidating, so you can click on
the question mark on the top right to receive a short tour of the windows
available in the app.

Once the dataset was imported and analysed according to your objective, you can
choose which visualisations to use from the Visualise window and press the
button "Launch iSEEtree" to create and customise the plots. After adjusting the
parameters of the different panels, the app might look something like this:

```{r screenplot2, echo=FALSE, out.width="100%"}
SCREENSHOT("screenshots/iSEEtree.png", delay = 20)
```

As before, you can click on the question mark on the top right to receive a tour
of the panels and their parameters. The best way to get familiar with the
interface is to experiment with the parameters below each panel.

# Resources

## Citation

We hope that miaDash will be useful for your research. Please use the following
information to cite the package and the overall approach. Thank you!

```{r citation}
## Citation info
citation("miaDash")
```

## Background Knowledge

miaDash originates from the joint effort of the R/Bioconductor community. It is
mainly based on the following software:

- [R](https://www.r-project.org/), statistical programming language [@core2024r]
- [mia](https://bioconductor.org/packages/release/bioc/html/mia.html), framework for microbiome data analysis [@borman2024mia]
- [iSEEtree](https://bioconductor.org/packages/release/bioc/html/iSEEtree.html), TreeSummarizedExperiment interactive explorer [@benedetti2024iseetree]
- [iSEE](https://bioconductor.org/packages/release/bioc/html/iSEE.html), SummarizedExperiment interactive explorer [@rue2018isee]
- [TreeSummarizedExperiment](https://bioconductor.org/packages/release/bioc/html/TreeSummarizedExperiment.htmlm), S4 container for hierarchical data [@huang2020treesummarizedexperiment]
- [shiny](https://cran.r-project.org/web/packages/shiny/index.html), web app development in R [@chang2024shiny]

## Help

You can reach us by one of the communication channels listed
[here](https://microbiome.github.io/OMA/docs/devel/pages/support.html). We are
happy to receive questions, suggestions as well as contributions. For the last
point, check the
[contributor guidelines](https://microbiome.github.io/mia/CONTRIBUTING.html).

# Reproducibility

R session information:

```{r reproduce, echo=FALSE}
# Session info
options(width = 120)
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/observers.R
\name{create_observers}
\alias{create_observers}
\alias{.create_import_observers}
\alias{.create_manipulate_observers}
\alias{.create_estimate_observers}
\alias{.update_observers}
\alias{.create_launch_observers}
\title{Observers}
\usage{
.create_import_observers(input, rObjects)

.create_manipulate_observers(input, rObjects)

.create_estimate_observers(input, rObjects)

.update_observers(input, session, rObjects)

.create_launch_observers(FUN, input, session, rObjects)
}
\arguments{
\item{input}{The Shiny input object from the server function.}

\item{rObjects}{A reactive list of values generated in the landing page.}
}
\value{
Observers are created in the server function in which this is called.
A \code{NULL} value is invisibly returned.
}
\description{
\code{.create_observers} and \code{.create_launch_observers} define the
server to import and build TreeSE objects and track the state of the Build
and Launch buttons.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constants.R
\docType{data}
\name{constants}
\alias{constants}
\alias{.miaDashDefaultPanels}
\alias{.miaDashOtherPanels}
\alias{.transformMethods}
\alias{.qualityMetrics}
\alias{.alphaMetrics}
\alias{.betaMetrics}
\alias{.betaMethods}
\alias{.clustMethods}
\alias{.DmmCriteria}
\title{Constants}
\format{
An object of class \code{character} of length 7.

An object of class \code{character} of length 9.

An object of class \code{character} of length 19.

An object of class \code{list} of length 4.

An object of class \code{character} of length 23.

An object of class \code{character} of length 4.

An object of class \code{character} of length 4.

An object of class \code{character} of length 4.

An object of class \code{character} of length 3.
}
\usage{
.miaDashDefaultPanels

.miaDashOtherPanels

.transformMethods

.qualityMetrics

.alphaMetrics

.betaMetrics

.betaMethods

.clustMethods

.DmmCriteria
}
\description{
Constant values used throughout the miaDash app.
}
\section{Panel layout}{

\describe{
\item{\code{.miaDashDefaultPanels}}{List of panel names in the default layout of miaDash.}
\item{\code{.miaDashOtherPanels}}{List of panel names not in the default layout of miaDash.}
\item{\code{.transformMethods}}{List of transformations applicable to assays.}
\item{\code{.alphaMetrics}}{List of alpha diversity metrics.}
\item{\code{.betaMetrics}}{List of beta diversity metrics.}
\item{\code{.betaMethods}}{List of beta diversity methods.}
\item{\code{.qualityMetrics}}{List of metrics for quality control.}
}
}

\author{
Giulio Benedetti
}
\keyword{datasets}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{utils}
\alias{utils}
\alias{.import_datasets}
\alias{.update_tse}
\alias{.print_message}
\alias{.set_optarg}
\alias{.check_formula}
\alias{.importBIOM}
\alias{.rownames2taxa}
\title{miaDash utilities}
\usage{
.import_datasets(selection)

.update_tse(tse, fun, fun.args = list())

.print_message(..., title = "Invalid input:")

.set_optarg(item, loader = NULL, alternative = NULL, ...)

.check_formula(form, tse)

.importBIOM(file, col.data = NULL, tree.file = NULL, ...)

.rownames2taxa(x)
}
\arguments{
\item{selection}{\code{Numeric vector}. A list of indices for the mia
datasets to return.}

\item{tse}{a
\code{\link[TreeSummarizedExperiment:TreeSummarizedExperiment-constructor]{TreeSummarizedExperiment}}
object.}

\item{fun}{\code{Function scalar}. Function to apply to \code{tse}.}

\item{fun.args}{\code{Named list}. A list of arguments to pass to \code{fun}.}

\item{...}{Either a series of strings to form the message returned by
\code{.print_message} or named arguments for \code{loader}.}

\item{title}{\code{Character scalar}. The title of the error message to
print. (Default: \code{"Invalid input:"})}

\item{item}{\code{Character scalar}. The file path from which the object
should be loaded.}

\item{loader}{\code{Function scalar}. The function to load \code{item}.
(Default: \code{NULL})}

\item{alternative}{an alternative output to return when at least one
\code{item} and \code{loader} are not defined. (Default: \code{NULL})}

\item{form}{\code{Character scalar} The formula to be checked.}
}
\value{
\itemize{
\item \code{.import_datasets}: returns the list of available mia datasets.
\item \code{.update_tse}: returns a TreeSE object after applying \code{fun}
with arguments \code{fun.args}. Eventual messages and errors are also
printed.
\item \code{.print_message}: returns a modalDialog with the error message
specified with \code{...} and titled \code{title}.
\item \code{.set_optarg}: returns an optional element for a TreeSE
constructor. The output is either an object located at the file path
\code{item} and loaded with \code{loader} or \code{alternative}.
\item \code{.check_formula}: returns \code{TRUE} or \code{FALSE} depending
whether or not all variables included in \code{form} are present in
\code{se} colData.
}
}
\description{
Helper functions to support the app functionality.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputs.R
\name{render_output}
\alias{render_output}
\alias{.render_overview}
\alias{.render_download}
\title{Outputs}
\usage{
.render_overview(output, rObjects)

.render_download(output, rObjects)
}
\arguments{
\item{output}{The Shiny output object from the server function.}
}
\value{
Adds a rendered item to \code{output}.
A \code{NULL} value is invisibly returned.
}
\description{
\code{.render_overview} and \code{.render_download} create the output of the
UI, which consists of the overview of the TreeSE object and the download
object, respectively.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/landing_page.R
\name{landing_page}
\alias{landing_page}
\alias{.landing_page}
\title{Landing page}
\usage{
.landing_page(FUN, input, output, session)
}
\value{
The UI is defined by the function. A \code{NULL} value is invisibly
returned.
}
\description{
\code{.landing_page} creates the landing page of miaDash, where TreeSE objects
can be built and iSEE can be launched.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miaDash.R
\name{miaDash}
\alias{miaDash}
\title{miaDash}
\usage{
miaDash()
}
\value{
An \code{\link[iSEE:iSEE]{iSEE}} app with a custom landing page to
build TreeSE objects and explore \link[mia:mia-datasets]{mia datasets}.
}
\description{
miaDash is a web app that provides an interface to build and explore
\code{\link[TreeSummarizedExperiment:TreeSummarizedExperiment-constructor]{TreeSummarizedExperiment}}
(TreeSE) objects by means of \link[iSEE:iSEE]{iSEE}.
}
\examples{
app <- miaDash()

if (interactive()) {
  shiny::runApp(app)
}

}
\seealso{
\link[iSEE:iSEE]{iSEE} \link[mia:mia]{mia}
\link[miaViz:miaViz]{miaViz}
}
